@@ -20,6 +20,7 @@ use strict;
 use warnings;
 use Carp;
 use LWP;
+use URI;
 use HTTP::Request::Common;
 use HTTP::Cookies;
 use XML::Simple;
@@ -31,7 +32,7 @@ use Error qw(:try);             # for web services verification (you may comment
 use Data::Dumper;               # dump hashes for debugging
 use File::Temp qw/ tempfile /;  # create temp files
 
-our $VERSION = '1.74';
+our $VERSION = '1.86';
 
 =head1 NAME
 
@@ -91,6 +92,12 @@ Set maximum number of HTTP redirects. Default is 0.
 
 Sets a proxy which is then used for http and https requests.
 
+ ex.: http://proxy.company.net:3128
+
+with authentication:
+
+ ex.: http://user:password@proxy.company.net:3128
+
 =item output_dir
 
 Output directory where all logfiles will go to. Defaults to current directory.
@@ -188,8 +195,6 @@ sub engine {
 
     $self->_processcasefile();
 
-    my $useragent = $self->_get_useragent();
-
     # write opening tags for STDOUT.
     $self->_writeinitialstdout();
 
@@ -242,7 +247,7 @@ sub engine {
             $repeat = $xmltestcases->{repeat};
         }
 
-        my $useragent = $self->_get_useragent();
+        my $useragent = $self->_get_useragent($xmltestcases->{case});
 
         for my $run_nr (1 .. $repeat) {
 
@@ -297,7 +302,6 @@ sub engine {
     my $endruntimer = time();
     $self->{'result'}->{'totalruntime'} = ( int( 1000 * ( $endruntimer - $startruntimer ) ) / 1000 );    #elapsed time rounded to thousandths
 
-
     # do return/cleanup tasks
     return $self->_finaltasks();
 }
@@ -309,14 +313,15 @@ sub _run_test_case {
 
     confess("no testcase!") unless defined $case;
 
-    $case->{'id'}          = 1 unless defined $case->{'id'};
-    $case->{'passedcount'} = 0;
-    $case->{'failedcount'} = 0;
-    $case->{'iswarning'}   = 0;
-    $case->{'iscritical'}  = 0;
-    $case->{'messages'}    = [];
+    # set some defaults
+    $case->{'id'}           = 1 unless defined $case->{'id'};
+    $case->{'passedcount'}  = 0;
+    $case->{'failedcount'}  = 0;
+    $case->{'iswarning'}    = 0;
+    $case->{'iscritical'}   = 0;
+    $case->{'messages'}     = [];
 
-    $useragent = $self->_get_useragent() unless defined $useragent;
+    $useragent = $self->_get_useragent({1 => $case}) unless defined $useragent;
 
     # don't do this if monitor is disabled in gui
     if($self->{'gui'} and $self->{'monitorenabledchkbx'} ne 'monitor_off') {
@@ -329,7 +334,16 @@ sub _run_test_case {
     for my $key (keys %{$case}) {
         $case->{$key} = $self->_convertbackxml($case->{$key}, $timestamp);
         next if $key eq 'errormessage';
-        $case->{$key} = $self->_convertbackxmlresult($case->{$key}, $timestamp);
+        $case->{$key} = $self->_convertbackxmlresult($case->{$key});
+    }
+
+    # replace host with realserverip in url and add http host header to useragent
+    if($self->{'config'}->{'realserverip'})
+    {
+        my($uri)=URI->new($case->{url});
+        my($host)=$uri->host();
+        $useragent->default_header('Host' => $uri->host());
+        $case->{url}=~s/\Q$host\E/$self->{'config'}->{'realserverip'}/mx;
     }
 
     if( $self->{'gui'} ) { $self->_gui_tc_descript($case); }
@@ -351,58 +365,67 @@ sub _run_test_case {
     push @{$case->{'messages'}}, { 'html' => "</td><td>" }; # HTML: next column
 
     my($latency,$request,$response);
-    if($case->{method}){
-        if(lc $case->{method} eq "get") {
-            ($latency,$request,$response) = $self->_httpget($useragent, $case);
-        }
-        elsif(lc $case->{method} eq "post") {
-            ($latency,$request,$response) = $self->_httppost($useragent, $case);
+    alarm($self->{'config'}->{'timeout'}+1); # timeout should be handled by LWP, but just in case...
+    eval {
+        local $SIG{ALRM} = sub { die("alarm") };
+        if($case->{method}){
+            if(lc $case->{method} eq "get") {
+                ($latency,$request,$response) = $self->_httpget($useragent, $case);
+            }
+            elsif(lc $case->{method} eq "post") {
+                ($latency,$request,$response) = $self->_httppost($useragent, $case);
+            }
+            else {
+                $self->_usage('ERROR: bad HTTP Request Method Type, you must use "get" or "post"');
+            }
         }
         else {
-            $self->_usage('ERROR: bad HTTP Request Method Type, you must use "get" or "post"');
+            ($latency,$request,$response) = $self->_httpget($useragent, $case);     # use "get" if no method is specified
         }
-    }
-    else {
-        ($latency,$request,$response) = $self->_httpget($useragent, $case);     # use "get" if no method is specified
-    }
-    $case->{'latency'}  = $latency;
-    $case->{'request'}  = $request->as_string();
-    $case->{'response'} = $response->as_string();
+    };
+    alarm(0);
+    if($@) {
+        $case->{'iscritical'} = 1;
+    } else {
+        $case->{'latency'}  = $latency;
+        $case->{'request'}  = $request->as_string();
+        $case->{'response'} = $response->as_string();
 
-    # verify result from http response
-    $self->_verify($response, $case);
+        # verify result from http response
+        $self->_verify($response, $case);
 
-    if($case->{verifypositivenext}) {
-        $self->{'verifylater'} = $case->{'verifypositivenext'};
-        $self->_out("Verify On Next Case: '".$case->{verifypositivenext}."' \n");
-        push @{$case->{'messages'}}, {'key' => 'verifypositivenext', 'value' => $case->{verifypositivenext}, 'html' => "Verify On Next Case: ".$case->{verifypositivenext}."<br />" };
-    }
+        if($case->{verifypositivenext}) {
+            $self->{'verifylater'} = $case->{'verifypositivenext'};
+            $self->_out("Verify On Next Case: '".$case->{verifypositivenext}."' \n");
+            push @{$case->{'messages'}}, {'key' => 'verifypositivenext', 'value' => $case->{verifypositivenext}, 'html' => "Verify On Next Case: ".$case->{verifypositivenext}."<br />" };
+        }
 
-    if($case->{verifynegativenext}) {
-        $self->{'verifylaterneg'} = $case->{'verifynegativenext'};
-        $self->_out("Verify Negative On Next Case: '".$case->{verifynegativenext}."' \n");
-        push @{$case->{'messages'}}, {'key' => 'verifynegativenext', 'value' => $case->{verifynegativenext}, 'html' => "Verify Negative On Next Case: ".$case->{verifynegativenext}."<br />" };
-    }
+        if($case->{verifynegativenext}) {
+            $self->{'verifylaterneg'} = $case->{'verifynegativenext'};
+            $self->_out("Verify Negative On Next Case: '".$case->{verifynegativenext}."' \n");
+            push @{$case->{'messages'}}, {'key' => 'verifynegativenext', 'value' => $case->{verifynegativenext}, 'html' => "Verify Negative On Next Case: ".$case->{verifynegativenext}."<br />" };
+        }
 
-    # write to http.log file
-    $self->_httplog($request, $response, $case);
+        # write to http.log file
+        $self->_httplog($request, $response, $case);
 
-    # send perf data to log file for plotting
-    $self->_plotlog($latency);
+        # send perf data to log file for plotting
+        $self->_plotlog($latency);
 
-    # call the external plotter to create a graph
-    $self->_plotit();
+        # call the external plotter to create a graph
+        $self->_plotit();
 
-    if( $self->{'gui'} ) {
-        $self->_gui_updatemontab();                 # update monitor with the newly rendered plot graph
-    }
+        if( $self->{'gui'} ) {
+            $self->_gui_updatemontab();                 # update monitor with the newly rendered plot graph
+        }
 
-    $self->_parseresponse($response, $case);        # grab string from response to send later
+        $self->_parseresponse($response, $case);        # grab string from response to send later
 
-    # make parsed results available in the errormessage
-    for my $key (keys %{$case}) {
-        next unless $key eq 'errormessage';
-        $case->{$key} = $self->_convertbackxmlresult($case->{$key}, $timestamp);
+        # make parsed results available in the errormessage
+        for my $key (keys %{$case}) {
+            next unless $key eq 'errormessage';
+            $case->{$key} = $self->_convertbackxmlresult($case->{$key});
+        }
     }
 
     push @{$case->{'messages'}}, { 'html' => "</td><td>\n" }; # HTML: next column
@@ -558,10 +581,19 @@ sub _run_test_case {
 
 ################################################################################
 sub _get_useragent {
-    my $self = shift;
+    my($self, $testcases) = @_;
 
-    # construct LWP object
-    my $useragent  = LWP::UserAgent->new;
+    # keepalive is required for ntml authentication but breaks
+    # https proxy support, so try determince which one we need
+    my $keepalive = 1;
+    if($testcases and $self->{'config'}->{'proxy'}) {
+        for my $nr (keys %{$testcases}) {
+            if($testcases->{$nr}->{'url'} =~ m/^https/gmx) {
+                $keepalive = 0;
+            }
+        }
+    }
+    my $useragent  = LWP::UserAgent->new(keep_alive=>$keepalive);
 
     # store cookies in our LWP object
     my($fh, $cookietempfilename) = tempfile(undef, UNLINK => 1);
@@ -570,6 +602,7 @@ sub _get_useragent {
                                                  file     => $cookietempfilename,
                                                  autosave => 1,
                                               ));
+    push @{$self->{'tmpfiles'}}, $cookietempfilename;
 
     # http useragent that will show up in webserver logs
     unless(defined $self->{'config'}->{'useragent'}) {
@@ -580,10 +613,40 @@ sub _get_useragent {
 
     # add proxy support if it is set in config.xml
     if( $self->{'config'}->{'proxy'} ) {
+        # try IO::Socket::SSL first
+        eval {
+            require IO::Socket::SSL;
+            IO::Socket::SSL->import();
+        };
         my $proxy = $self->{'config'}->{'proxy'};
-        $proxy    =~ s/^http:\/\///mx;
-        $useragent->proxy([qw( http )], "http://".$proxy);
-        $ENV{'HTTPS_PROXY'} = "http://".$proxy;
+        $proxy    =~ s/^http(s|):\/\///mx;
+        # http just works
+        $useragent->proxy('http', 'http://'.$proxy);
+        # authentication?
+        my $proxyuser = '';
+        my $proxypass = '';
+        if($proxy =~ s/^(.*?):(.*?)@(.*)$/$3/gmx) {
+            $proxyuser = $1;
+            $proxypass = $2;
+        }
+        # ssl depends on which class we have
+        if($INC{'IO/Socket/SSL.pm'}) {
+            $ENV{PERL_NET_HTTPS_SSL_SOCKET_CLASS} = "IO::Socket::SSL";
+            if($proxypass) {
+                $proxy = $proxyuser.':'.$proxypass.'@'.$proxy;
+            }
+            my $con_proxy = 'connect://'.$proxy;
+            $useragent->proxy('https', $con_proxy);
+        } else {
+            # ssl proxy only works this way, see http://community.activestate.com/forum-topic/lwp-https-requests-proxy
+            $ENV{PERL_NET_HTTPS_SSL_SOCKET_CLASS}   = "Net::SSL";
+            $ENV{PERL_LWP_SSL_VERIFY_HOSTNAME}      = 0;
+            $ENV{HTTPS_PROXY}                       = $proxy;
+            $ENV{HTTPS_PROXY_USERNAME}              = $proxyuser;
+            $ENV{HTTPS_PROXY_PASSWORD}              = $proxypass;
+            # env proxy breaks the ssl proxy above
+            #$useragent->env_proxy();
+        }
     }
 
     # don't follow redirects unless set by config
@@ -622,6 +685,7 @@ sub _set_defaults {
         'reporttype'                => 'standard',
         'output_dir'                => './',
         'nooutput'                  => undef,
+        'realserverip'              => '',
         'baseurl'                   => '',
         'baseurl1'                  => '',
         'baseurl2'                  => '',
@@ -630,6 +694,7 @@ sub _set_defaults {
         'globalhttplog'             => 'no',
         'proxy'                     => '',
         'timeout'                   => 180,
+        'tmpfiles'                  => [],
     };
     $self->{'exit_codes'}         = {
         'UNKNOWN'  => 3,
@@ -934,10 +999,13 @@ sub _httppost_xml {
     my @xmlbody = <$xmlbody>;    # read the file into an array
     close($xmlbody);
 
+    # Get the XML input file to use PARSEDRESULT and substitute the contents
+    my $content = $self->_convertbackxmlresult(join( " ", @xmlbody ));
+
     $self->_out("POST Request: ".$case->{url}."\n");
     $request = new HTTP::Request( 'POST', $case->{url} );
     $request->content_type($case->{posttype});
-    $request->content( join( " ", @xmlbody ) );    # load the contents of the file into the request body
+    $request->content( $content );    # load the contents of the file into the request body
 
     ($latency,$request,$response) = $self->_http_defaults($request, $useragent, $case);
 
@@ -1237,7 +1305,7 @@ sub _parseresponse {
             $self->{'parsedresult'}->{$type} = $1;
         }
         ## use critic
-        else {
+        elsif(!defined $case->{'parsewarning'} or $case->{'parsewarning'}) {
             push @{$case->{'messages'}}, {'key' => $type.'-success', 'value' => 'false', 'html' => "<span class=\"fail\">Failed Parseresult, cannot find</span> $leftboundary(.*?)$rightboundary" };
             $self->_out("Failed Parseresult, cannot find $leftboundary(*)$rightboundary\n");
             $case->{'iswarning'} = 1;
@@ -1303,7 +1371,7 @@ sub _read_config_xml {
     foreach (@configlines) {
 
         for my $key (
-            qw/baseurl baseurl1 baseurl2 gnuplot proxy timeout output_dir
+            qw/realserverip baseurl baseurl1 baseurl2 gnuplot proxy timeout output_dir
             globaltimeout globalhttplog standaloneplot max_redirect
             break_on_errors useragent/
           )
@@ -1430,6 +1498,7 @@ sub _convtestcases {
     my @xmltoconvert;
 
     my ( $fh, $tempfilename ) = tempfile();
+    push @{$self->{'tmpfiles'}}, $tempfilename;
     my $filename = $currentcasefile;
     open( my $xmltoconvert, '<', $filename )
       or $self->_usage("ERROR: Failed to read test case file: ".$filename.": ".$!);
@@ -1480,6 +1549,7 @@ sub _convertbackxml {
     $string =~ s~{AMPERSAND}~&~gmx;
     $string =~ s~{LESSTHAN}~<~gmx;
     $string =~ s~{TIMESTAMP}~$timestamp~gmx;
+    $string =~ s~{REALSERVERIP}~$self->{'config'}->{realserverip}~gmx;
     $string =~ s~{BASEURL}~$self->{'config'}->{baseurl}~gmx;
     $string =~ s~{BASEURL1}~$self->{'config'}->{baseurl1}~gmx;
     $string =~ s~{BASEURL2}~$self->{'config'}->{baseurl2}~gmx;
@@ -1489,7 +1559,7 @@ sub _convertbackxml {
 ################################################################################
 # converts replaced xml with parsed result
 sub _convertbackxmlresult {
-    my ( $self, $string, $timestamp ) = @_;
+    my ( $self, $string) = @_;
     return unless defined $string;
     $string =~ s~\{PARSEDRESULT\}~$self->{'parsedresult'}->{'parseresponse'}~gmx if defined $self->{'parsedresult'}->{'parseresponse'};
     for my $x (1..5) {
@@ -1643,6 +1713,8 @@ plot \"$self->{'config'}->{'output_dir'}plot.log\" using 1:7 title \"Response Ti
 sub _finaltasks {
     my $self        = shift;
 
+    $self->_clean_tmp_files();
+
     if ( $self->{'gui'} ) { $self->_gui_stop(); }
 
     # we suppress most logging when running in a plugin mode
@@ -1669,13 +1741,13 @@ sub _finaltasks {
                 $crit = $self->{'config'}->{globaltimeout};
             }
             my $lastid = 0;
-            my $perfdata = '|time='.$self->{'result'}->{'totalruntime'}.';0;'.$crit.';0;0';
+            my $perfdata = '|time='.$self->{'result'}->{'totalruntime'}.'s;0;'.$crit.';0;0';
             for my $file (@{$self->{'result'}->{'files'}}) {
                 for my $case (@{$file->{'cases'}}) {
                     my $warn   = $case->{'warning'}  || 0;
                     my $crit   = $case->{'critical'} || 0;
                     my $label  = $case->{'label'}    || 'case'.$case->{'id'};
-                    $perfdata .= ' '.$label.'='.$case->{'latency'}.';'.$warn.';'.$crit.';0;0';
+                    $perfdata .= ' '.$label.'='.$case->{'latency'}.'s;'.$warn.';'.$crit.';0;0';
                     $lastid = $case->{'id'};
                 }
             }
@@ -1683,7 +1755,7 @@ sub _finaltasks {
             for my $nr (1..($self->{'result'}->{'casecount'} - $self->{'result'}->{'totalruncount'})) {
                 $lastid++;
                 my $label  = 'case'.$lastid;
-                $perfdata .= ' '.$label.'=0;0;0;0;0';
+                $perfdata .= ' '.$label.'=0s;0;0;0;0';
             }
 
             my($rc,$message);
@@ -1886,6 +1958,16 @@ EOB
     exit 3;
 }
 
+################################################################################
+# remove any tmp files
+sub _clean_tmp_files {
+    my($self) = @_;
+    for my $tmpfile (@{$self->{'tmpfiles'}}) {
+        unlink($tmpfile);
+    }
+    return;
+}
+
 =head1 EXAMPLES
 
 =head2 example test case
